import win32process
import platform
import typing
import pygetwindow as gw


def get_pid_from_hwnd(hwnd):
    """
    Get the process ID given the handle of a window.

    Args:
        hwnd (int or gw.Win32Window): The handle of the window. If it is an instance of gw.Win32Window, its handle will be extracted.

    Returns:
        int or None: The process ID of the window, or None if an error occurred.
    """
    if not isinstance(hwnd, int):
        assert isinstance(hwnd, gw.Win32Window)
        hwnd = hwnd._hWnd

    try:
        _, pid = win32process.GetWindowThreadProcessId(hwnd)
        return pid
    except Exception as e:
        print(f"Error: {e}")
        return None


def get_process_wnds(processname: str) -> typing.List[gw.BaseWindow]:
    """
    Get a list of windows associated with a specific process name.

    Args:
        processname (str): The name of the process to search for.

    Returns:
        List[gw.Window]: A list of windows associated with the specified process name.
    """
    if not platform.system() == "Windows":
        raise NotImplementedError

    procs = []
    import psutil

    for proc in psutil.process_iter():
        if proc.name().startswith(processname):
            procs.append(proc)

    procIds = [proc.pid for proc in procs]

    wnds = []
    for wnd in gw.getAllWindows():
        wnd: gw.Win32Window

        _, winpid = win32process.GetWindowThreadProcessId(wnd._hWnd)

        if winpid in procIds:
            wnds.append(wnd)
    return wnds
